C     -----------------------------------------------------------------------
C     
C     J 2 E D I
C     
C     PROGRAME TO REORDER MT DATA FROM JONES FORMAT TO SEG/EDI FORMAT
C     -----------------------------------------------------------------------
C     a.g.jones,  ottawa,   Feb, 1989
c     
c     Modification history:
c     
c     agj: Feb94: modified to handle single sites
c     jim: Oct94: stop those incerssant azimuth prompts
c     jim: Feb95: only use R type if Z type not found in data file
c     jim: Mar95: handle gds only sites using WRITEMEAS, WRITEZROT
c     agj: Nov95: reads coord info from full J-format file
c                 write out a j2edi.cfg file first time
c     jim: Aug96: multi-site or single site per edifile
c 2.0 agj: Mar97: doesn't require TF file to have same freqs as GB MT file
c                 uses first MT freq to set freq to start TF
c 2.1 agj: Apr97: Corrected lines for "dplon" to have parentheses outside
c                 the whole -1*  (from Pascal Tarits)
c 2.2 agj: Mar98: Wrote formatted output instead of * format
c                 Changed GT_MISDAT to avoid overflows on PC
c                 Checked dates in correct sequence (mm/dd/yy)
c 2.3 xavi:Jun99: Correct an error on the calculation of the longitudes
c                 and latitudes. Change the geo2utm subroutine to one
c                 that calculates the zone by itself (j2edi no longer
c                 asks for this).
c 2.4 agj: Sep99: checks whether rotated transfer functions when from different 
c                 file and azimuth to MT-GB responses
c 2.5 agj: Oct99: supports M__ types (output from Geotools)
c          Dec01: writes out ZXX and ZYY values as 1.E-10 for GB data
c                 and values of 1.E-9 for variances
c          Feb02: sets TZX & TYX to 1.E-10 when missing and 1.E-9
c                 for variances
c          Jan03: sets large values (>TMAX) of TZX & TZY to EMPTY
c 2.6      Jan04: if TZ data in different azimuth from MT-GB data, then
c                 TZ data rotated
c 2.7      Sep09: added today's date to edi output file
c                 commented out DSUN dependencies
c                 added RHOPHA option in cfg file to preferentially look for RHO and PHA first
c-----------------------------------------------------------------------
c     You must provide an tfread subroutine to get your impedances and
c     geomag transfer functions into j2edi.
c
c     tfread provides vectors of real & imag impedances, their error and weight
c     for a certain type: ZXX,ZXY,ZYX or ZYY
c
c     mtread is used to read the rho & phase data instead of the impedances
c     if they are not found in the data file
c
c     mtread provides vectors of rho and phase and their max & min values
c     ( & weight info in wrho and wpha which may be ignored )
c     for a certain type, eg RXY, RYX, see initialization of types above
c
c     in both tfread mand mtread the '*' parameter is used to signal the 
c     subroutine the data file is already attached to fortran unit ind
c
c     ir_isnan() is a function supplied by Sun to check for NaN's
c
c-----------------------------------------------------------------------

      implicit none

c-----------------------------------------------------------------------

      include 'ctrlblk.inc'
      include 'pi.inc'
      include 'misdat.inc'
      include 'io.inc'


c...Geotools default missing data marker = 1.E+32
c   written out in "EMPTY" in header
      real GT_MISDAT
      parameter ( GT_MISDAT = 1.E+32 )

      integer MAXF, MAXS
      parameter ( MAXF  =  200 ) !max. no. freqs
      parameter ( MAXS  = 1000 ) !max no of sites

      real TMAX
      parameter ( TMAX = 2.0 )   !maximum permitted value of TZX or TZY

c-----------------------------------------------------------------------
      real period(MAXF), per1, pertf(MAXF),
     &     rho(MAXF),    pha(MAXF), 
     &     rhomax(MAXF), rhomin(MAXF), 
     &     phamax(MAXF), phamin(MAXF), 
     &     wrho(MAXF),   wpha(MAXF),
     &     tfr(MAXF),    tfi(MAXF),    
     &     tferr(MAXF),  wtf(MAXF), 
     &     tfvar(MAXF), 
     &     north, east, north0, east0, zone
     
      real txr(MAXF), txi(MAXF), txvar(MAXF), 
     &     tyr(MAXF), tyi(MAXF), tyvar(MAXF)
      complex tzx, tzy, tzxr, tzyr, rottf
      real theta, var


      complex z, convr2z, z1, z2, z3, z4

      character fedi(MAXS)*80, type*3, line*80, stattf*6
      character*3 types(6), mistypes(6)
      character*4 ver, cfgver
      character*6 stat, stats(MAXS)
      character*80 fin, fins(MAXS)

      character*16 acqby, adate, yname, date, prov, area, country,
     &     refloc

      character*24 date_string

      logical RHOPHA, DISCARD, GB, TF, TFINMT, Z_FOUND
      logical WRITEMEAS, WRITEFZROT, CFG
      logical MULTIEDI, WRTEDIHEAD, DEGEAST, MODELDATA
      logical TZROT

      real lat, lon, elev, lon1, lat1
      real lats(MAXS), lons(MAXS), elevs(MAXS)
      integer londeg, lonmin, lonsec, latdeg, latmin, latsec
      integer istop, izone, nfile, ntype, nstat, i, istat, l, it,
     &         ieast, inorth, iheight, icall, n, ntf, itfs, ii, ierror
      integer lacqby, ladate, lyname, ldate, lcountry, lprov,
     &        lfedi, larea, lrefloc
      integer ldot
      integer missing_types, itype

      real    alat, alon, dplat, dplon, dnorth, deast
      real    zrot, azimmt, azimtz

      integer lnblnk


c#ifdef SUN
c      integer ir_isnan
c#endif
c-------------------------------------------------------------------------------
c     initialize

      call fdate(date_string)

c-----------------------------------------------------------------------
c...version number

      ver = 'v2.7'

c-----------------------------------------------------------------------

      iprint = 0
      call iin( 'Give print level', iprint )

      fedi(1) = 'multi'
      izone = -999

      if( iprint.ge.0 ) then
        write(*,*)' '
        write(*,*)'           Welcome to J2EDI'
        write(*,*)'              ver = ', ver
        write(*,*)'             MAXF = ', MAXF
        write(*,*)'             MAXS = ', MAXS
        write(*,*)' '
      endif
      
c----------------------------- ------------------------------------------
c     open input files

      open(unit=ind2,file='j2edi.cfg',status='old',err=1)
      write(*,2001)'Opened J2EDI configuration file >j2edi.cfg'
      CFG = .TRUE.
      goto 10
 1    write(*,*)'J2EDI configuration file not found, opening >',
     &     'j2edi.cfg< to write params into'
      open(unit=outd2,file='j2edi.cfg',status='new')
      CFG = .FALSE.

      nfile = 0


 10   continue

      if( CFG ) then
        read( ind2,'(10x,a)')  cfgver
        if( cfgver.ne.ver ) then
          write(*,*) ' '
          write(*,*) 'j2edi.cfg is an old version: ver = '//cfgver
          write(*,*) '....delete j2edi.cfg and re-run to create '//
     &               'new cfg file....'
          write(*,*) 'Do NOT just change the version number in the '//
     &               'existing cfg file!!!'
          write(*,*) ' '
          stop
        endif
        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'DEGEAST   line >',line(1:lnblnk(line))
        read( line,'(10x,l1)') DEGEAST
        
        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'RHOPHA    line >',line(1:lnblnk(line))
        read( line,'(10x,l1)') RHOPHA

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'DISCARD   line >',line(1:lnblnk(line))
        read( line,'(10x,l1)') DISCARD

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'GB        line >',line(1:lnblnk(line))
        read( line,'(10x,l1)') GB

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'MODELDATA line >',line(1:lnblnk(line))
        read( line,'(10x,l1)') MODELDATA

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'TF        line >',line(1:lnblnk(line))
        read( line,'(10x,l1)') TF

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'TFINMT    line >',line(1:lnblnk(line))
        read( line,'(10x,l1)') TFINMT

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'ACQBY     line >',line(1:lnblnk(line))
        read(line,'(10x, a16)') acqby
        call trunc( acqby, lacqby )

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'ADATE     line >',line(1:lnblnk(line))
        read(line,'(10x, a16)') adate
20      call trunc( adate, ladate )
        call chkdat( adate, ladate, ierror )
        if( ierror.ne.0 ) then
          write(*,'(a,$)')'Give correct acquisition date (mm/dd/yy) >'
          read(*,'(a)') adate
          goto 20
        endif

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'YNAME     line >',line(1:lnblnk(line))
        read(line,'(10x, a16)') yname
        call trunc( yname, lyname )

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'DATE      line >',line(1:lnblnk(line))
        read(line,'(10x, a16)') date
25      call trunc( date, ldate )
        call chkdat( date, ldate, ierror)
        if( ierror.ne.0 ) then
          write(*,'(a,$)')'Give correct date (mm/dd/yy) >'
          read(*,'(a)') date
          goto 25
        endif

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'COUNTRY   line >',line(1:lnblnk(line))
        read(line,'(10x, a16)') country
        call trunc( country, lcountry )

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'PROV      line >',line(1:lnblnk(line))
        read(line,'(10x, a16)') prov
        call trunc( prov, lprov )

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'AREA      line >',line(1:lnblnk(line))
        read(line,'(10x, a16)') area
        call trunc( area, larea )

        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'MULTIEDI  line >',line(1:lnblnk(line))
        read(line,'(10x,l1)') MULTIEDI
        if( .not.MULTIEDI ) then
          write(*,*)'Multiple sites in one EDI file no longer ',
     &              'supported as WinGLink cannot handle them'
          MULTIEDI = .TRUE.
        endif
        
        read(ind2,'(a)') line
        if(iprint.ge.1)write(*,*)'FEDI      line >',line(1:lnblnk(line))
        read(line,'(10x, a80)') fedi(1)
        call trunc( fedi(1), lfedi )

      else

        write(outd2,'(a,a)') 'VERSION  =', ver

        DEGEAST = .FALSE.
        call lin( 'Are longitudes in degrees EAST of Greenwich',
     &              DEGEAST )
        write(outd2,'(a,l1)') 'DEGEAST  =', DEGEAST

        RHOPHA = .FALSE.
        call lin( 'Are RHOs/PHAs preferred over impedances', RHOPHA )
        write(outd2,'(a,l1)') 'RHOPHA   =',RHOPHA

        DISCARD = .FALSE.
        call lin( 'Are negative rhos to be discarded', DISCARD )
        write(outd2,'(a,l1)') 'DISCARD  =',DISCARD

        GB = .FALSE.
        call lin( 'Are data from Groom-Bailey (no RXX or RYY)', GB )
        write(outd2,'(a,l1)') 'GBDATA   =', GB

        MODELDATA = .FALSE.
        call lin( 'Are these model data (types M__)', MODELDATA )
        write(outd2,'(a,l1)') 'MODELDATA=', MODELDATA

        TF = .TRUE.
        call lin( 'Are vertical field transfer functions to be '//
     &            'read in', TF )
        write(outd2,'(a,l1)') 'TFDATA   =', TF

        TFINMT = .TRUE.
        if( TF ) then
          call lin( 'Are transfer functions in MT .dat file', TFINMT )
        endif
        write(outd2,'(a,l1)') 'TFINMT   =', TFINMT

        call getline( 'Give acquired by info', acqby, lacqby )

        write(outd2,'(2a)') 'ACQBY    =',acqby
31      call getline( 'Acquisition date (mm/dd/yy)', adate, ladate )
        call chkdat( adate, ladate, ierror )
        if( ierror.gt.0 ) goto 31
        write(outd2,'(2a)') 'ADATE    =',adate
        call getline( 'Your name', yname, lyname )
        write(outd2,'(2a)') 'FILEBY   =',yname
32      call getline( 'Todays date (mm/dd/yy)', date, ldate )
        call chkdat( date, ldate, ierror )
        if( ierror.gt.0 ) goto 32
        write(outd2,'(2a)') 'FILEDATE =',date
        call getline( 'Country', country, lcountry )
        write(outd2,'(2a)') 'COUNTRY  =',country
        call getline( 'Give province/stat', prov, lprov )
        write(outd2,'(2a)') 'STATE    =',prov
        call getline( 'Give area', area, larea )
        write(outd2,'(2a)') 'AREA     =',area

c...MULTIEDI is only one supported now as WinGLink cannot read in more than one site per EDI file    
        MULTIEDI = .TRUE.
c        call lin('Write an edi file for each .dat file?', MULTIEDI)
        if(.not.MULTIEDI)then
          call tin( 'Give filename for output file', fedi(1) )
          call ftype( fedi(1), 'edi' )
          write(*,*)'First .dat file will be the REFLOC.  All site',
     $               ' offsets are calculated from it.'
        endif
        write(outd2,'(a,l1)') 'MULTIEDI =', MULTIEDI
        call trunc( fedi(1), lfedi )
        write(outd2,'(a,a)')  'EDIFILE  =', fedi(1)(1:lfedi)
      endif

      if( GB ) then
        ntype = 2
        types(1) = 'RXY'
        types(2) = 'RYX'
      elseif( MODELDATA ) then
        ntype = 2
        types(1) = 'MXY'
        types(2) = 'MYX'
      else
        ntype = 4
        types(1) = 'RXX'
        types(2) = 'RXY'
        types(3) = 'RYX'
        types(4) = 'RYY'
      endif

      if( TF ) then
        if( MODELDATA ) then
          types(ntype+1) = 'MZX'
          types(ntype+2) = 'MZY'
        else
          types(ntype+1) = 'TZX'
          types(ntype+2) = 'TZY'
        endif
        ntype = ntype+2
      endif

      nfile = 0
 100  fin = '.dat'
      call iofile( 2, 'Give data filename (STOP to stop)', ind1, 
     &     fin, *1111 )
      continue
      nfile = nfile+1
      fins(nfile) = fin

      nstat = nfile
      ldot = index(fin, '.')
      if( ldot.eq.0 ) ldot = 6
      stat  = fin(1:min(ldot-1,6))
      if( iprint.ge.1 ) then
        write(*,*)'file in >',fin(1:lnblnk(fin)),'   stat >',stat
      endif
      stats(nstat) = stat

c...store lat & longinfo
      call getstathd (ind1,zrot,lat,lon,elev)
      lats(nstat)  = lat
      lons(nstat)  = lon
      elevs(nstat) = elev
      close(ind1)
      goto 100

      
1111  write(*,*)'All data files opened: nfile =', nfile

c...sift thrugh lats and longs, ensure all are valid
      istop = 0
      do i = 1, nstat
        if(lats(i).eq.-999..or.lons(i).eq.-999) then
          write(*,*)' Error in lat or long for site ', stats(i)
          istop = 1
        endif
      enddo
      if(istop.eq.1) then
        stop
      endif
c     
c-----------------------------------------------------------------------
c     HEAD & INFO block
c
      do istat = 1, nstat
        stat = stats(istat)
         
        if( MULTIEDI ) then
          fedi(istat) = fins(istat)(1:lnblnk(fins(istat))-4)//'.edi'
          open(unit=outd1, file=fedi(istat), status='NEW')    
        elseif (istat.eq.1) then
          open(unit=outd1, file=fedi(1), status='NEW')    
        endif
        call iofile( 2, ' ', ind1, fins(istat), *9999 )

        WRTEDIHEAD = MULTIEDI .or. (.not.MULTIEDI .and. istat.eq.1)
        if(WRTEDIHEAD) then
          write(outd1,2001)'>HEAD'
          write(outd1,2001)' '
c...DATAID
          write(outd1,2001) 'DATAID="'//stat(1:lnblnk(stat))//'"'
c...ACQBY                                         
          lacqby = min( lacqby, 16)
          write(outd1,2001) 'ACQBY="'//acqby(1:lacqby)//'"'
c...ACQDATE
          ladate = min( ladate, 16)
          write(outd1,2001) 'ACQDATE='//adate(1:ladate)
c...FILEBY
          lyname = min( lyname, 16)
          write(outd1,2001) 'FILEBY="'//yname(1:lyname)//'"'
c...FILEDATE
          ldate = min( ldate, 16)
          write(outd1,2001) 'FILEDATE='//date(1:ldate)
c...COUNTRY
          lcountry = min( lcountry, 16)
          write(outd1,2001) 'COUNTRY='//country(1:lcountry)
c...PROVINCE/STATE
          lprov = min( lprov, 16)
          write(outd1,2001) 'STATE="'//prov(1:lprov)//'"'
c...AREA/COUNTY
          larea = min( larea, 16)
          write(outd1,2001) 'COUNTY="'//area(1:larea)//'"'
c...BINDATA
c          write(outd1,2001) 'BINDATA="FALSE"'
c...STDVERS
          write(outd1,2001) 'STDVERS=2.3'
c...PROGVERS
          write(outd1,2001) 'PROGVERS="JONES 2.3"'
c...PROGDATE
          write(outd1,2001) 'PROGDATE=03/14/98'
c...EMPTY
          write(outd1,2003) 'EMPTY=',GT_MISDAT
            
          write(outd1,2001)' '
          write(outd1,2001)' '
          write(outd1,2001)'>INFO'
          write(outd1,2001)'MAXINFO=1000'
          write(outd1,2001)' '         
        endif

c        INFO block
c
        write(outd1,'(80a1)') '-'
        write(outd1,'(a)') ' '
        write(outd1,'(a)') 'j2edi: version '//ver
        write(outd1,'(a)') 'date : '//date_string
        write(outd1,'(a)') ' '
c...get information block from data file
        write(outd1,'(a)') 'Information block from data file...'
        write(outd1,'(a)') fins(istat)
        read(  ind1,'(a)') line
        do while( line(1:1).eq.'#'.or.line(1:1).eq.'>' )
          call trunc( line, l)
          it = index(line,'>')
          if( it.ne.0 ) then
            line(it:it) = ' '
          endif
          write(outd1,'(a)') line(2:l)
          read(  ind1,'(a)') line
        enddo
        write(outd1,'(a)') ' '
        backspace ind1

        write(outd1,'(80a1)') '-'
        if(MULTIEDI) then
          close(outd1)
        endif
        close(ind1)
      enddo

c-----------------------------------------------------------------------
c  REFLOC

      do istat = 1, nstat

        stat = stats(istat)
        if(MULTIEDI) then
          open(unit=outd1, file=fedi(istat), access='APPEND',err=9999)    
        endif
        call iofile( 2, ' ', ind1, fins(istat), *9999 )

        WRTEDIHEAD = MULTIEDI.or.(.not.MULTIEDI.and.istat.eq.1)
        if(WRTEDIHEAD) then

          lat = lats(istat)
          lon = lons(istat)

          if( iprint.ge.1 ) then
            write(*,*)'Lat/long of REFLOC=', lat, lon
          endif

          lon1 = lon
          lat1 = lat

          if( lon.lt.0. ) lon = -lon
          if( lat.lt.0. ) lat = -lat

          londeg = ifix(lon)
          lonmin = ifix(lon*60.)   - londeg*60
          lonsec = ifix(lon*3600.) - lonmin*60 - londeg*3600
          latdeg = ifix(lat)
          latmin = ifix(lat*60.)   - latdeg*60
          latsec = ifix(lat*3600.) - latmin*60 - latdeg*3600

          if( lon1.lt.0. ) londeg = -londeg
          if( lat1.lt.0. ) latdeg = -latdeg

c-----------------------------------------------------------------------
c     DEFINEMEAS block

          write(outd1,2001)' '
          write(outd1,2001) '>=DEFINEMEAS'

          write(outd1,2001) 'MAXRUN=999'
          write(outd1,2001) 'MAXMEAS=1024'
          write(outd1,2001) 'UNITS=M'
          write(outd1,2001) 'REFTYPE=CART'

c     call getline( 'Give reference location', refloc, lrefloc)
          refloc = stat
          lrefloc = min(lrefloc,16)
c...Geotools suddenly doesn't like REFLOC
c          write(outd1,2001) 'REFLOC='//refloc
          write(outd1,'(a,i3.2,a1,i2.2,a1,i2.2)') 'REFLAT=', 
     &           latdeg,':',latmin,':',latsec            
          write(outd1,'(a,i4.3,a1,i2.2,a1,i2.2)') 'REFLONG=', 
     &           londeg,':',lonmin,':',lonsec

          write(outd1,2003) 'REFELEV=', elevs(istat)
          write(outd1,2001) ' '

          dplat = sign(-1,latdeg)*
     &           (float(abs(latdeg)) + float(latmin)/60.
     &                     + float(latsec)/3600.)
          dplon = sign(-1,londeg)*
     &           (float(abs(londeg)) + float(lonmin)/60.
     &                       + float(lonsec)/3600.)
          if( iprint.ge.3 ) then
            write(*,*)'Latitude and longitude =', dplat,dplon
          endif
!          if(izone.eq.-999) then
!            izone = 12
!            call iin('UTM zone ?',izone)
!            dcm = 183 - izone * 6
!          endif
          zone=999.0
          call geo2utm(dplat,dplon,zone,dnorth,deast)
          east0  = nint(deast)
          north0 = nint(dnorth)
          if( iprint.ge.3 ) then
            write(*,*)'east0, north0 =', east0,north0
          endif
c     --- endif for if(WRTEDIHEAD)
        endif

c-----------------------------------------------------------------------
c     EMEAS & HMEAS blocks for each site

        lat = lats(istat)
        lon = lons(istat)

        if( iprint.ge.1 ) then
          write(*,*)'Lat/long=', lat, lon
        endif

        lon1 = lon
        lat1 = lat

        if( lon.lt.0. ) lon = -lon
        if( lat.lt.0. ) lat = -lat

        londeg = ifix(lon)
        lonmin = ifix(lon*60.)   - londeg*60
        lonsec = ifix(lon*3600.) - lonmin*60 - londeg*3600
        latdeg = ifix(lat)
        latmin = ifix(lat*60.)   - latdeg*60
        latsec = ifix(lat*3600.) - latmin*60 - latdeg*3600

        if( lon1.lt.0. ) londeg = -londeg
        if( lat1.lt.0. ) latdeg = -latdeg

        dplat = sign(-1,latdeg)*
     &           (float(abs(latdeg)) + float(latmin)/60.
     &                     + float(latsec)/3600.)
c...v2.1 correction begins
        dplon = sign(-1,londeg)*
     &           (float(abs(londeg)) + float(lonmin)/60.
     &                     + float(lonsec)/3600.)
c...v2.1 correction ends

        zone=999.
        call geo2utm(dplat,dplon,zone,dnorth,deast)
        east = nint(deast)
        north = nint(dnorth)

        ieast  =  east -  east0
        inorth = north - north0
        iheight = 0

        write(outd1,1001)'>EMEAS ID=', istat*10+1, istat, ' CHTYPE=EX',
     &        ' X=',inorth-50, ' Y=',ieast, ' Z=',iheight, 
     &        ' X2=',inorth+50, ' Y2=',ieast
        write(outd1,1001)'>EMEAS ID=', istat*10+2, istat, ' CHTYPE=EY',
     &        ' X=',inorth, ' Y=',ieast-50, ' Z=',iheight, 
     &        ' X2=',inorth, ' Y2=',ieast+50
        write(outd1,1001)'>HMEAS ID=', istat*10+3, istat, ' CHTYPE=HX',
     &        ' X=',inorth, ' Y=',ieast, ' Z=',iheight, ' AZM=0'
        write(outd1,1001)'>HMEAS ID=', istat*10+4, istat, ' CHTYPE=HY',
     &        ' X=',inorth, ' Y=',ieast, ' Z=',iheight, ' AZM=90'
        write(outd1,1001)'>HMEAS ID=', istat*10+5, istat, ' CHTYPE=HZ',
     &        ' X=',inorth, ' Y=',ieast, ' Z=',iheight, ' AZM=0'
        write(outd1,*) ' '
1001    format(a,i4.4,'.',i3.3,a, 5(a,i7) )

        if(MULTIEDI) then
          close(outd1)
        endif
        close(ind1)
      enddo

c-----------------------------------------------------------------------
      do istat = 1, nstat
        stat = stats(istat)
         
        if(MULTIEDI) then
          open(unit=outd1,file=fedi(istat),access='APPEND',err=9999)    
        endif
        call iofile( 2, ' ', ind1, fins(istat), *9999 )

        WRITEMEAS  = .TRUE.
        WRITEFZROT = .TRUE.
        missing_types = 0

        write(*,*)'>>>>>>>> station ----------->',stat

        do 1000 it = 1, ntype
          type = types(it)
          if( iprint.ge.1) write(*,*)'Doing type...',type

          if( type(1:1).eq.'R' 
     &        .or. type.eq.'MXY' .or. type.eq.'MYX') then
            icall = icall+1

c             --- attempt to read z first if unsuccessful, use mtread
            n = -1
            if( .not.RHOPHA ) then
              type(1:1)='Z'
              call tfread ( '*', stat, type, n, period, tfr, tfi,
     &              tferr,wtf)
              if( n.gt.MAXF ) then
                write(*,*)'Number of data too large: MAXF =', MAXF
                stop
              endif
              if(n.le.0) then
                Z_FOUND=.FALSE.
                write(*,*)'Unable to find ',type,' as impedances - ',
     &                   'reading in as rhos & phases'
              else
                Z_FOUND=.TRUE.
                if(iprint.ge.1) write(*,2001)'Read MT '//type//
     &                             ' impedances in...  n =', n
c     ---  convert Z from S.I. to field units for GEOTOOLS
                per1 = period(1)
                do i = 1, n
                  if( tferr(i).le.0. .or. wtf(i).lt.0. ) then
                    if(iprint.ge.1) write(*,*)'Setting period',i,
     &                                        '  to GT_MISDAT'
                    tfr(i)   = GT_MISDAT
                    tfi(i)   = GT_MISDAT
                    tferr(i) = GT_MISDAT
                    wtf (i)  = GT_MISDAT
                  else
                    tfr(i)  = tfr(i)/ (4.*PI*1.e-4)
                    tfi(i)  = tfi(i)/ (4.*PI*1.e-4)
                    tfvar(i)= (tferr(i)/(4.*PI*1.e-4))**2
                  endif
                enddo
              endif
            endif

            if( .not.Z_FOUND ) then            
              type(1:1)='R'
              call mtread( '*', stat, type, n, period, rho, pha,
     &                 rhomax, rhomin, phamax, phamin, wrho, wpha )
              if( iprint.ge.1 ) write(*,2001)'Read MT '//type//
     &                            ' rhos and phases in...  n =', n
              if(n.le.0) then
                write(*,*)' WARNING: R/Z',type(2:3),' not found'
                write(*,*)'          setting type to missing'
                missing_types = missing_types + 1
                mistypes(missing_types) = type
                goto 1000
              endif
              
            else
            endif

c     ---on first call for this site, then write MTSECT, FREQ & ZROT blocks

            if( WRITEMEAS ) then
              write(outd1,2001)' '
              write(outd1,2001) '>=MTSECT'
              write(outd1,2001) 'NFREQ=', n
              write(outd1,2001) 'SECTID="'//stat//'"'
              write(outd1,1004) 'EX=', istat*10+1, istat,
     &                           'EY=', istat*10+2, istat,
     &                           'HX=', istat*10+3, istat,
     &                           'HY=', istat*10+4, istat,
     &                           'HZ=', istat*10+5, istat

              WRITEMEAS=.FALSE.
            endif

            if(WRITEFZROT) then
              write(outd1,2001)' '
              write(outd1,2001)'>FREQ ORDER=INC  //',n
              write(outd1,2002)(1./period(i),i=n,1,-1)
              write(outd1,2001)' '
              call getstathd (ind1,zrot,alat,alon,elev)
              if(zrot.eq.-999.) then
                zrot = 0.
                call rin (
     $                    'X-direction (deg from T.N.) for station '//
     &                    stat //'?', zrot)
              endif
              write(outd1,2001)'>ZROT   //', n
              write(outd1,2002)(zrot,i=1,n)
              WRITEFZROT=.FALSE.
            endif


c...convert data from rho only if Z not found
            if( .not. Z_FOUND) then
              do i = 1, n
                if( rho(i).lt.0.     .and. 
     &               rho(i).ne.MISDAT .and. 
     &                    .not.DISCARD      )  rho(i) = -rho(i)
     
                if( rho(i).lt.0. .or. pha(i).eq.MISDAT ) then
                  if( iprint.ge.1 ) write(*,*)'Setting period',i,
     &                                               '  to GT_MISDAT'
                  tfr(i)    = GT_MISDAT
                  tfi(i)    = GT_MISDAT
                  tfvar(i)  = GT_MISDAT
                else
                  z = convr2z( rho(i), pha(i), period(i) )
c...convert Z from S.I. to field units for GEOTOOLS (and WinGLink?) ---NEEDS TESTING!!!
                  z = z / (4.*PI*1.e-4)
                  tfr(i) =  real(z)
                  tfi(i) = aimag(z)

                  if( rhomax(i).lt.0.     .or. 
     &                rhomin(i).lt.0.     .or.
     &                phamax(i).eq.MISDAT .or.
     $                phamin(i).eq.MISDAT      ) then
                    tfvar(i) = GT_MISDAT
                  else
                    z1 = convr2z( rhomax(i), pha(i), period(i) )
                    z2 = convr2z( rhomin(i), pha(i), period(i) )
                    z3 = convr2z( rho(i), phamax(i), period(i) )
                    z4 = convr2z( rho(i), phamin(i), period(i) )
c     ---  convert Z's from S.I. to field units for GEOTOOLS
                    z1 = z1 / (4.*PI*1.e-4)
                    z2 = z2 / (4.*PI*1.e-4)
                    z3 = z3 / (4.*PI*1.e-4)
                    z4 = z4 / (4.*PI*1.e-4)
                    tfvar(i) = cabs(z4-z3) * cabs(z1-z2) / 4.
                  endif
                endif
              enddo                   
            endif

            write(outd1,2001) '>Z'//type(2:3)//'R  //', n
            write(outd1,2002) (tfr(i),i=n,1,-1)        
            write(outd1,2001) '>Z'//type(2:3)//'I  //', n
            write(outd1,2002) (tfi(i),i=n,1,-1)
            write(outd1,2001) '>Z'//type(2:3)//'.VAR  //', n
            write(outd1,2002) (tfvar(i),i=n,1,-1)        

c...for GB files set ZXX & ZYY to 0.0 and variances to 1.0
c   ...for WinGLink, set to 1.E-8            
            if( GB .and. it.eq.2 ) then
              write(outd1,2001) '>ZXXR  //',n
              write(outd1,2002) (1.E-10,i=1,n)   
              write(outd1,2001) '>ZXXI  //',n
              write(outd1,2002) (1.E-10,i=1,n)   
              write(outd1,2001) '>ZXX.VAR  //',n
              write(outd1,2002) (1.E-9,i=1,n)
              write(outd1,2001) '>ZYYR  //',n
              write(outd1,2002) (1.E-10,i=1,n)   
              write(outd1,2001) '>ZYYI  //',n
              write(outd1,2002) (1.E-10,i=1,n)   
              write(outd1,2001) '>ZYY.VAR  //',n
              write(outd1,2002) (1.E-9,i=1,n)

              if( missing_types.ne.0 ) then
                do itype = 1, missing_types
                  write(outd1,2001) '>Z'//mistypes(itype)(2:3)
     &                              //'R  //',n
                  write(outd1,2002) (0.0,i=1,n)   
                  write(outd1,2001) '>Z'//mistypes(itype)(2:3)
     &                              //'I  //',n
                  write(outd1,2002) (0.0,i=1,n)   
                  write(outd1,2001) '>Z'//mistypes(itype)(2:3)
     &                              //'.VAR  //',n
                  write(outd1,2002) (0.0,i=1,n)   
                enddo
              endif

            endif  

          else  
c---- type is not 'R??'

            if( TFINMT ) then
              call tfread( '*', stat, type, n, period, tfr, tfi, 
     &                 tferr, wtf )
              if(iprint.ge.1) write(*,2001) 'Read '//type//
     &                           ' transfer functions in... n =',n
              if(iprint.ge.2) then
	        write(*,*)'Real(TF):'
                write(*,*)(tfr(i),i=1,n)
		write(*,*)'Imag(TF):'
                write(*,*)(tfi(i),i=1,n)
	      endif

c...set zero values to 1.E-10 (for WinGLink!)
              do i = 1, n
                if( tfr(i).eq.0. .and. tfi(i).eq.0 ) then
                  tfr(i)   = 1.e-10
                  tfi(i)   = 1.e-10
                  tferr(i) = 1.e-9
                  wtf(i)   = 0.
                endif
              enddo

              do i = 1, n
                if( 
c#ifdef SUN
c     &              (ir_isnan(tfr(i)).ne.0)   .or.
c     &              (ir_isnan(tfi(i)).ne.0)   .or.
c     &              (ir_isnan(tferr(i)).ne.0) .or.
c#endif
     &              tferr(i).le.0.            .or.
     &              wtf(i).lt.0.                   ) then
                  if(iprint.ge.1)write(*,*)'Setting period', i,
     &                                      '  to GT_MISDAT'
                  tfr(i)   = GT_MISDAT
                  tfi(i)   = GT_MISDAT
                  tferr(i) = GT_MISDAT
                  wtf (i)  = GT_MISDAT
                endif
              enddo
            else

c..reads in TF responses from other file than MT-GB one has to check azimuth
c  it = 3 for TZX for GB data
c  it = 5 for TZX for non-GB data

              if( it.eq.3 .or. it.eq.5 ) then
                if( GB ) then
                  fin = fin(1:6)//fin(8:)
                endif
                call iofile( 2, 'Give tf data filename (STOP to stop)',
     &                         ind1, fin, *9999 )

                stattf = fin(1:6)
                call shiftup(stattf)

c...check that the station names are the same
                if( stat.ne.stattf ) then
                  write(*,*)'Error in files: mt stat=',stat,
     &                       '    tf stat=', stattf
                  stop
                endif

c...check that the rotation angles are the same
                azimmt = zrot
                call getstathd (ind1,azimtz,lat,lon,elev)
                if( azimmt.eq.azimtz ) then
                  TZROT = .FALSE.
                else
                  TZROT = .TRUE.
                  write(*,*)'TZs in different azimuth from MT responses'
                  write(*,*)'MT azimuth:', azimmt
                  write(*,*)'TZ azimuth:', azimtz
                  write(*,*)'TZs will be rotated by ', azimmt-azimtz,
     &                      '   degrees'
                endif

              endif
	      
              call tfread( '*', stat, type, ntf, pertf, tfr, tfi, 
     &                       tferr, wtf )
              if(iprint.ge.1) write(*,'(a)') 'Read '//type//
     &                               ' transfer functions in....'
              if(iprint.ge.2) then
	        write(*,*)'Real(TF):'
                write(*,*)(tfr(i),i=1,n)
		write(*,*)'Imag(TF):'
                write(*,*)(tfi(i),i=1,n)
		write(*,*)'Err(TF):'
		write(*,*)(tferr(i),i=1,n)
		write(*,*)'Weight(TF):'
		write(*,*)(wtf(i),i=1,n)
	      endif
	      
c...set zero values to 1.E-10 (for WinGLink!)
              do i = 1, ntf
                if( tfr(i).eq.0. .and. tfi(i).eq.0 ) then
                  tfr(i)   = 1.e-10
                  tfi(i)   = 1.e-10
                  tferr(i) = 1.e-9
                  wtf(i)   = 0.
                endif
              enddo
		
c...set to GT_MISDAT for values >TMAX
              do i = 1, ntf
                if( abs(tfr(i)).gt.TMAX .or. abs(tfi(i)).gt.TMAX ) then
                  tfr(i)   = GT_MISDAT
                  tfi(i)   = GT_MISDAT
                  tferr(i) = GT_MISDAT
                  wtf(i)   = 0.
                endif
              enddo

              if( n.ne.ntf ) then
                write(*,*)'No. of data in files do not match: ',
     &                      'mt n=', n, '    tf n=', ntf
                itfs = 0
                do i = 1, ntf
                  if( per1*0.95.lt.pertf(i) .and. 
     &                per1*1.05.gt.pertf(i) ) itfs = i
                enddo

                if( itfs.eq.0 ) then
                  write(*,2001) 'Could not match first period of MT'//
     &                           'data file with periods in TF file'
                  write(*,*) '1st period of MT data file =', per1
                  stop
                else
                  write(*,*)'Matching period found for period =',
     &                        itfs, pertf(itfs)
                endif
              else
                itfs = 1
              endif
              do ii = itfs, n+itfs-1
                i = ii - itfs + 1
                if( tferr(i).le.0. .or. wtf(i).lt.0. ) then
                    tfr(i)   = GT_MISDAT
                    tfi(i)   = GT_MISDAT
                    tferr(i) = GT_MISDAT
                    wtf(i)   = GT_MISDAT
                else
                  tfr(i)   = tfr(ii)
                  tfi(i)   = tfi(ii)
                  tferr(i) = tferr(ii)
                  wtf(i)   = wtf(ii)
                endif
              enddo
            endif
c            write(*,*)'After ii loop'
c            write(*,*)'Real(TF):'
c            write(*,*)(tfr(i),i=1,n)
c            write(*,*)'Imag(TF):'
c            write(*,*)(tfi(i),i=1,n)

            if(WRITEMEAS) then
              write(outd1,2001)' '
              write(outd1,2001) '>=MTSECT'
              write(outd1,2001) 'NFREQ=', n
              write(outd1,2001) 'SECTID="'//stat//'"'
              write(outd1,1004) 'EX=', istat*10+1, istat,
     &                           'EY=', istat*10+2, istat,
     &                           'HX=', istat*10+3, istat,
     &                           'HY=', istat*10+4, istat,
     &                           'HZ=', istat*10+5, istat
              WRITEMEAS=.FALSE.
            endif

            if(WRITEFZROT)then
              write(outd1,2001)' '
              write(outd1,2001)'>FREQ ORDER=INC  //',n
              write(outd1,2002)(1./period(i),i=n,1,-1)
              write(outd1,2001)' '
              call getstathd (ind1,zrot,alat,alon,elev)
              if(zrot.eq.-999.) then
                zrot = 0.
                call rin('X-direction (deg from T.N.) for station '//
     &                    stat //'?', zrot)
              endif
              write(outd1,2001)'>ZROT   //', n
              write(outd1,2002)(zrot,i=1,n)
              WRITEFZROT=.FALSE.
            endif

c...set missing data markers to the Geotools missing data marker
            do i = 1, n
              if( tfr(i)  .eq. MISDAT ) tfr(i)   = GT_MISDAT
              if( tfi(i)  .eq. MISDAT ) tfi(i)   = GT_MISDAT
              if( tferr(i).eq. MISDAT .or. 
     &            tferr(i).eq. GT_MISDAT     ) then 
                tfvar(i) = GT_MISDAT
              else
                tfvar(i) = tferr(i)**2.
              endif
            enddo
    
            if( n.gt.0 ) then
              if( TZROT ) then
                if( type(3:3).eq.'X' ) then
                  do i = 1, n
                    txr(i)   = tfr(i)
                    txi(i)   = tfi(i)
                    txvar(i) = tfvar(i)
                  enddo
                else
                  do i = 1, n
                    tyr(i)   = tfr(i)
                    tyi(i)   = tfi(i)
                    tyvar(i) = tfvar(i)
                  enddo	  
                endif
              else
                write(outd1,2001) '>T'//type(3:3)//'R.EXP  //', n
                write(outd1,2002) (tfr(i),i=n,1,-1)        
                write(outd1,2001) '>T'//type(3:3)//'I.EXP  //', n
                write(outd1,2002) (tfi(i),i=n,1,-1)        
                write(outd1,2001) '>T'//type(3:3)//'VAR.EXP  //',n
                write(outd1,2002) (tfvar(i),i=n,1,-1)
              endif
            endif
          endif
c...end of itype loop - all types read in and written out
1000    continue


c...special case for rotating TZ transfer functions
        if( TZROT ) then
	  theta = azimmt-azimtz
	  do i = 1, n
	    tzx = cmplx( txr(i), txi(i) )
	    tzy = cmplx( tyr(i), tyi(i) )
	    tzxr = rottf( tzx,  tzy, theta )
	    tzyr = rottf( tzy, -tzx, theta )
	    txr(i) =  real(tzxr)
	    txi(i) = aimag(tzxr)
	    tyr(i) =  real(tzyr)
	    tyi(i) = aimag(tzyr)
c...set variances to geometric mean of the two values
c...THIS IS A HORRIBLE KLUDGE!!!
            var = sqrt(txvar(i)*tyvar(i))
	    txvar(i) = var
	    tyvar(i) = var
	  enddo
          write(outd1,2001) '>TXR.EXP  //', n
          write(outd1,2002) (txr(i),i=n,1,-1)        
          write(outd1,2001) '>TXI.EXP  //', n
          write(outd1,2002) (txi(i),i=n,1,-1)        
          write(outd1,2001) '>TXVAR.EXP  //',n
          write(outd1,2002) (txvar(i),i=n,1,-1)
          write(outd1,2001) '>TYR.EXP  //', n
          write(outd1,2002) (tyr(i),i=n,1,-1)        
          write(outd1,2001) '>TYI.EXP  //', n
          write(outd1,2002) (tyi(i),i=n,1,-1)        
          write(outd1,2001) '>TYVAR.EXP  //',n
          write(outd1,2002) (tyvar(i),i=n,1,-1)
	endif


        if( MULTIEDI) then
          write(outd1,'(a)')'>END'
          close(outd1) 
          write(*,*)'Wrote ',fedi(istat)
        endif         
        close(ind1)
      enddo
      
      if(.not.MULTIEDI) then
         write(outd1,*)'>END'
         close(outd1) 
         write(*,*)'Wrote ',fedi
      endif
      stop 

 9999 continue
      stop


1002  format(5(a,i7))
1003  format(3(a,i7),a)
1004  format(5(a,i4.4,'.',i3.3,1x))

2001  format(a,i5)
2002  format(5(1x,g14.7))
2003  format(a,g14.7)

      end


c-----------------------------------------------------------------------
      subroutine getline( prompt, line, lline )
      character prompt*(*), line*(*)

      line = ' '
      call tin( prompt, line )
      call trunc( line, lline )
      call shiftup(line)
      return
      end 

      function sum( a, n )
      real a(*)
      sum = 0.
      do i = 1, n
        sum = sum + a(i)
      enddo
      return
      end

c-----------------------------------------------------------------------
      subroutine chkdat( date, ldate, ierror )
      implicit none
      
      include 'ctrlblk.inc'
      
      integer ldate, ierror, mm, dd, yy, ipos1, ipos2
      character date*(*)

c...checks that date in mm/dd/yy format
c   on error does a RETURN 1

      if( iprint.ge.10 ) write(*,'(3a,i5)') 'chkdat: entered with'//
     &                           'date=', date,   'ldate =', ldate
      
      ipos1 = index(date,'/')
      ipos2 = index(date(ipos1+1:),'/') + ipos1

      read( date(1:ipos1-1),* ) mm
      read( date(ipos1+1:ipos2-1),*) dd
      read( date(ipos2+1:),*) yy

      if( mm.lt.1 .or. mm.gt.12 ) then
        write(*,'(2a)')'Erroneous month in date >', mm
        ierror = 1
        return
      endif

      if( dd.lt.1 .or. dd.gt.31 ) then
        write(*,'(2a)')'Erroneous day in date >', dd
        ierror = 2
        return
      endif

      ierror = 0
      return
      end




      subroutine geo2utm( Lat, Long, longorig, Northing, Easting )
c
c	FORTRAN routine based on a C routine taken from the Web
c
c  Xavi: 06/02/99 : Inclusion of this subroutine in version v2.3
c
c
      real lat, long, northing, easting, longorig

      real*8 aa, eccSquared, k0, pi
      data aa/6378206.4/
      data eccSquared/0.00676866/
      data k0/0.9996/
      data pi/3.14159265d0/

      real*8 LongOrigin
      real*8 eccPrimeSquared
      real*8 N, T, C, A, M

      deg2rad = pi/180.d0

      LongOrigin = longorig

      if( longorig.eq.999.0 ) then
        if( Long.eq.0) then
          LongOrigin = -3.d0
        else
          LongOrigin = dble( float(int(long/6.0))*6.0 +
     &      3.0*float(int(long/6.0))/abs(float(int(long/6.0))) )
        endif
      endif

      longorig = LongOrigin

      eccPrimeSquared = (eccSquared)/(1-eccSquared)


      N = aa/sqrt(1-eccSquared*sin(Lat*deg2rad)*sin(Lat*deg2rad))
      T = tan(Lat*deg2rad)*tan(Lat*deg2rad)
      C = eccPrimeSquared*cos(Lat*deg2rad)*cos(Lat*deg2rad)
      A = cos(Lat*deg2rad)*(Long-LongOrigin)*deg2rad

      M = 111132.0894*Lat - 16216.94*sin(2.*Lat*deg2rad)
     &         + 17.21*sin(4.*Lat*deg2rad) - 0.02*sin(6.*Lat*deg2rad)

      Easting = k0*N*(A+(1-T+C)*A*A*A/6 +
     &         (5-18*T+T*T+72*C-58*eccPrimeSquared)*A*A*A*A*A/120)
     &                                  + 500000.0

      Northing =
     &   k0*(M+N*tan(Lat*deg2rad)*(A*A/2+(5-T+9*C+4*C*C)*A*A*A*A/24
     &                           +
     & (61-58*T+T*T+600*C-330*eccPrimeSquared)*A*A*A*A*A*A/720))

      return
      end
      